﻿using System;
using System.IO;
using System.Linq;
using System.Net;
using System.Security;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Xml.Linq;
using Microsoft.WindowsAzure.ServiceRuntime;

namespace ProductionOrStagingRole
{
    public static class ServiceManagement
    {
        public static string GetDeploymentSlot(string subscriptionId, string cloudServiceName, string managementCertificateThumbprint)
        {
            return GetDeploymentSlot(
                new Guid(subscriptionId), 
                cloudServiceName,
                managementCertificateThumbprint,
                RoleEnvironment.DeploymentId);
        }

        public static string GetDeploymentSlot(Guid subscriptionId, string cloudServiceName, string managementCertificateThumbprint)
        {
            return GetDeploymentSlot(
                subscriptionId, 
                cloudServiceName, 
                managementCertificateThumbprint,
                RoleEnvironment.DeploymentId);
        }

        public static string GetDeploymentSlot(string subscriptionId, string cloudServiceName,
            string managementCertificateThumbprint, string deploymentId)
        {
            return GetDeploymentSlot(
                new Guid(subscriptionId),
                cloudServiceName,
                managementCertificateThumbprint,
                deploymentId);
        }

        public static string GetDeploymentSlot(Guid subscriptionId, string cloudServiceName, 
            string managementCertificateThumbprint, string deploymentId)
        {
            var cloudServiceProperties = GetCloudServiceProperties(
                subscriptionId,
                cloudServiceName,
                managementCertificateThumbprint);

            XNamespace aw = "http://schemas.microsoft.com/windowsazure";

            var deploymentSlot = cloudServiceProperties
                .Descendants(aw + "Deployment")
                .Select(d => new
                {
                    id = d.Element(aw + "PrivateID"), 
                    slot = d.Element(aw + "DeploymentSlot")
                })
                .Where(d => d.id != null && d.id.Value == deploymentId)
                .Select(d => d.slot)
                .FirstOrDefault();

            return deploymentSlot != null ? deploymentSlot.Value : "Unknown";
        }

        private static XDocument GetCloudServiceProperties(Guid subscriptionId, string cloudServiceName, string managementCertificateThumbprint)
        {
            XDocument responseDocument;

            var uri = new Uri(String.Format(
                "https://management.core.windows.net/{0}/services/hostedservices/{1}?embed-detail=true",
                subscriptionId,
                cloudServiceName));
            var request = CreateHttpWebRequest(uri, "GET", managementCertificateThumbprint);

            using (var response = request.GetResponse())
            {
                using (var responseStream = response.GetResponseStream())
                {
                    if (responseStream == null)
                        throw new Exception("Server did not respond.");

                    using (var readStream = new StreamReader(responseStream, Encoding.UTF8))
                    {
                        responseDocument = XDocument.Load(readStream);

                        readStream.Close();
                    }
                    responseStream.Close();
                }
                response.Close();
            }

            return responseDocument;
        }

        private static X509Certificate2 GetX509Certificate2(String thumbprint)
        {
            X509Certificate2 certificate;
            var store = new X509Store(StoreName.My, StoreLocation.CurrentUser);
            try
            {
                store.Open(OpenFlags.ReadOnly);
                var certCollection = store.Certificates.Find(
                    X509FindType.FindByThumbprint,
                    thumbprint,
                    false);

                if (certCollection.Count == 0)
                    throw new Exception("No certificate found containing thumbprint " + thumbprint);

                certificate = certCollection[0];
            }
            finally
            {
                store.Close();
            }
            return certificate;
        }

        private static HttpWebRequest CreateHttpWebRequest(Uri uri, String httpWebRequestMethod, string managementCertificateThumbprint)
        {
            var x509Certificate2 = GetX509Certificate2(managementCertificateThumbprint);

            var httpWebRequest = (HttpWebRequest)WebRequest.Create(uri);
            httpWebRequest.Method = httpWebRequestMethod;
            httpWebRequest.Headers.Add("x-ms-version", "2009-10-01");
            httpWebRequest.ClientCertificates.Add(x509Certificate2);
            httpWebRequest.ContentType = "application/xml";

            return httpWebRequest;
        }
    }
}